#!/bin/sh

. ../../program_error.sh

# This is a test file to run the PB test for MMPBSA

if [ -z "$AMBERHOME" ]; then
   echo "Error: AMBERHOME must be set!"
   exit 1
fi

# Define executable
if [ -z "$DO_PARALLEL" ]; then
   EXE=$AMBERHOME/bin/MMPBSA.py
else
   if [ `$DO_PARALLEL ../../numprocs` -gt 2 ]; then
      echo "Cannot have more than 2 processors for this test."
      exit 0
   fi
   EXE=$AMBERHOME/bin/MMPBSA.py.MPI
fi

# Write input file
cat > mmpbsa.in << EOF
MMPBSA input file for running PB
&general
   startframe=1, endframe=2, interval=1,
   keep_files=0, debug_printlevel=2
/
&pb
   istrng=0.1,
/
EOF

# Execute the program
$DO_PARALLEL $EXE -O -i mmpbsa.in \
                     -sp ../EstRAL_Files/sol.top \
                     -cp ../EstRAL_Files/com.top \
                     -rp ../EstRAL_Files/rec.top \
                     -lp ../EstRAL_Files/lig.top \
                     -eo energies.csv \
                     -y ../EstRAL_Files/test.mdcrd > mmpbsa.out 2>&1 || error

../../dacdif -t 3 -a 0.2 FINAL_RESULTS_MMPBSA.dat.save FINAL_RESULTS_MMPBSA.dat
# Strip out the commas so we can use relative error comparisons
sed -i -e "s@,@ @g" energies.csv
../../dacdif -r 1e-3 energies.csv.save energies.csv

/bin/rm -f mmpbsa.in mmpbsa.out *-e
